@@ -16,6 +16,18 @@ from httplib import HTTPException
 
 from eventlet import Timeout, sleep
 
+def fake_get_keystoneclient_2_0(os_options, exc=None):
+    def fake_get_keystoneclient_2_0(auth_url,
+                                    user,
+                                    key,
+                                    actual_os_options):
+        if exc:
+            raise exc('test')
+        if actual_os_options != os_options:
+            return "", None
+        return ("http://url/", "token")
+    return fake_get_keystoneclient_2_0
+
 
 def fake_http_connect(*code_iter, **kwargs):
 
